/*******************************************************************************
* File Name: ProgramControl_TunerHelper.c
* Version 2.0
*
* Description:
*  This file provides the source code of the Tuner helper APIs for the CapSense CSD 
*  component.
*
* Note:
*
********************************************************************************
* Copyright 2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "ProgramControl_TunerHelper.h"

#if (ProgramControl_TUNER_API_GENERATE)
    #include "SCB.h"
    #include "SCB_EZI2C.h"
    volatile ProgramControl_MAILBOXES ProgramControl_mailboxesComm;
    extern uint8 ProgramControl_sensorOnMask[(((ProgramControl_TOTAL_SENSOR_COUNT - 1u) / 8u) + 1u)];
#endif  /* (ProgramControl_TUNER_API_GENERATE) */


/*******************************************************************************
* Function Name: ProgramControl_TunerStart
********************************************************************************
*
* Summary:
*  Initializes the CapSense CSD component and EzI2C communication component to use
*  a mailbox data structure for communication with the Tuner GUI.
*  Start the scanning, after initialization is complete.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global Variables:
*  None
*
* Side Effects:
*  None
*
*******************************************************************************/
void ProgramControl_TunerStart(void)
{
    #if (ProgramControl_TUNER_API_GENERATE)
        
        /* Init mbx and quick check */
        ProgramControl_InitMailbox(&ProgramControl_mailboxesComm.csdMailbox);
        ProgramControl_mailboxesComm.numMailBoxes = ProgramControl_DEFAULT_MAILBOXES_NUMBER;
        
        /* Start CapSense and baselines */
        ProgramControl_Start();
        
        /* Initialize baselines */ 
        ProgramControl_InitializeEnabledBaselines();
        ProgramControl_InitializeEnabledBaselines();
        
        /* Start EzI2C, clears buf pointers */
        SCB_Start();
        
        /* Setup EzI2C buffers */
        SCB_EzI2CSetBuffer1(sizeof(ProgramControl_mailboxesComm), sizeof(ProgramControl_mailboxesComm),
                                        (volatile uint8 *) &ProgramControl_mailboxesComm);
        
        /* Starts scan all enabled sensors */
        ProgramControl_ScanEnabledWidgets();
    
    #endif  /* (ProgramControl_TUNER_API_GENERATE) */
}


/*******************************************************************************
* Function Name: ProgramControl_TunerComm
********************************************************************************
*
* Summary:
*  This function is blocking. It waits till scanning loop is completed and applies
*  the new parameters from the Tuner GUI if available (the manual tuning mode only). Updates
*  the enabled baselines and state of widgets. Waits while the Tuner GUI reports that 
*  the mailbox content  could be modified. Then loads the report data into the outbox 
*  and sets a busy flag. Starts a new scanning loop.
*  
* Parameters:
*  None
*
* Return:
*  None
*
* Global Variables:
*  None
*
* Side Effects:
*  None
*
*******************************************************************************/
void ProgramControl_TunerComm(void)
{
    #if (ProgramControl_TUNER_API_GENERATE)
        if (0u == ProgramControl_IsBusy())
        {   
            /* Apply new settings */
            #if (ProgramControl_TUNING_METHOD == ProgramControl__TUNING_MANUAL)
                ProgramControl_ReadMessage(&ProgramControl_mailboxesComm.csdMailbox);
            #endif  /* (ProgramControl_TUNING_METHOD == ProgramControl__TUNING_MANUAL) */

            /* Update all baselines and process all widgets */
            ProgramControl_UpdateEnabledBaselines();
            ProgramControl_ProcessAllWidgets(&ProgramControl_mailboxesComm.csdMailbox.outbox);
            ProgramControl_PostMessage(&ProgramControl_mailboxesComm.csdMailbox);

            /* Enable EZI2C interrupts, after scan complete */
            SCB_EnableInt();

            while((ProgramControl_mailboxesComm.csdMailbox.type != ProgramControl_TYPE_ID) || \
			      (0u != (SCB_EzI2CGetActivity() & SCB_EZI2C_STATUS_BUSY))){;}
            
            /* Disable EZI2C interrupts, while scanning */
            SCB_DisableInt();
            
            /* Start scan all sensors */
            ProgramControl_ScanEnabledWidgets();
        }
    #endif /* (ProgramControl_TUNER_API_GENERATE) */

}


#if (ProgramControl_TUNER_API_GENERATE)
    /*******************************************************************************
    * Function Name: ProgramControl_ProcessAllWidgets
    ********************************************************************************
    *
    * Summary:
    *  Calls the required functions to update all the widgets' state:
    *   - ProgramControl_GetCentroidPos() - calls only if linear sliders are
    *     available.
    *   - ProgramControl_GetRadialCentroidPos() - calls only if the  radial slider is
    *     available.
    *   - ProgramControl_GetTouchCentroidPos() - calls only if the  touchpad slider 
    *     available.
    *   - ProgramControl_CheckIsAnyWidgetActive();
    *  The results of operations are copied to OUTBOX.
    *   
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    * Global Variables:
    *  ProgramControl_OUTBOX outbox - the structure which is used as an output 
    *  buffer to report data to the Tuner GUI.
    *  Update fields:
    *    - position[];
    *    - OnMask[];
	*
	* Global Variables:
	*  None
	*
	* Side Effects:
	*  None
	*
    *******************************************************************************/
    void ProgramControl_ProcessAllWidgets(volatile ProgramControl_OUTBOX *outbox)
    {
        uint8 i = 0u;
		#if (ProgramControl_TOTAL_MATRIX_BUTTONS_COUNT)
        	uint8 mbPosition[2u];
		#endif /* (ProgramControl_TOTAL_MATRIX_BUTTONS_COUNT) */

        #if (ProgramControl_TOTAL_TOUCH_PADS_COUNT)
            uint16 pos[2];
        #endif  /* (ProgramControl_TOTAL_TOUCH_PADS_COUNT) */
        
        #if ( (ProgramControl_TOTAL_RADIAL_SLIDERS_COUNT) || (ProgramControl_TOTAL_TOUCH_PADS_COUNT) || \
              (ProgramControl_TOTAL_MATRIX_BUTTONS_COUNT) )
            uint8 widgetCnt;
        #endif  /* ((ProgramControl_TOTAL_RADIAL_SLIDERS_COUNT) || (ProgramControl_TOTAL_TOUCH_PADS_COUNT)) || 
                *   (ProgramControl_TOTAL_MATRIX_BUTTONS_COUNT)
                */
        
        /* Calculate widget with centroids */
        #if (ProgramControl_TOTAL_LINEAR_SLIDERS_COUNT)
            for(; i < ProgramControl_TOTAL_LINEAR_SLIDERS_COUNT; i++)
            {
                outbox->position[i] = (uint16)ProgramControl_GetCentroidPos((uint32)i);
            }
        #endif /* (ProgramControl_TOTAL_LINEAR_SLIDERS_COUNT) */
        
        #if (ProgramControl_TOTAL_RADIAL_SLIDERS_COUNT)
            widgetCnt = i;
            for(; i < (widgetCnt + ProgramControl_TOTAL_RADIAL_SLIDERS_COUNT); i++)
            {
                outbox->position[i] = (uint16)ProgramControl_GetRadialCentroidPos((uint32)i);
            }
        #endif /* (ProgramControl_TOTAL_RADIAL_SLIDERS_COUNT) */
        
        #if (ProgramControl_TOTAL_TOUCH_PADS_COUNT)
            widgetCnt = i;
            for(; i < (widgetCnt + (ProgramControl_TOTAL_TOUCH_PADS_COUNT * 2u)); i=i+2u)
            {
                if(ProgramControl_GetTouchCentroidPos((uint32)i, pos) == 0u)
                {
                    outbox->position[i] = 0xFFFFu;
                    outbox->position[i+1u] = 0xFFFFu;
                }
                else
                {
                    outbox->position[i] = (uint16) pos[0u];
                    outbox->position[i+1u] =(uint16) pos[1u];
                }
            }
        #endif /* (ProgramControl_TOTAL_TOUCH_PADS_COUNT) */

        #if (ProgramControl_TOTAL_MATRIX_BUTTONS_COUNT)
            i += ProgramControl_TOTAL_BUTTONS_COUNT;
            widgetCnt = 0u;
            for(; widgetCnt < (ProgramControl_TOTAL_MATRIX_BUTTONS_COUNT * 2u); widgetCnt += 2u)
            {
                if(ProgramControl_GetMatrixButtonPos((uint32)i, mbPosition) == 0u)
                {
                    outbox->mb_position[widgetCnt] = 0xFFu;
                    outbox->mb_position[widgetCnt+1u] = 0xFFu;
                }
				else
				{
					outbox->mb_position[widgetCnt] = mbPosition[0u];
					outbox->mb_position[widgetCnt+1u] = mbPosition[1u];
				}
                i += 2u;
            }
        #endif /* (ProgramControl_TOTAL_MATRIX_BUTTONS_COUNT) */

        /* Update On/Off State */
        (void)ProgramControl_CheckIsAnyWidgetActive();

        /* Copy OnMask */
        for(i=0u; i < ProgramControl_TOTAL_SENSOR_MASK_COUNT; i++)
        {
            outbox->onMask[i]  = ProgramControl_sensorOnMask[i];
        }
    }


#endif /* (ProgramControl_TUNER_API_GENERATE) */


/* [] END OF FILE */
