/*******************************************************************************
* File Name: ProgramControl_TunerHelper.c
* Version 2.10
*
* Description:
*  This file provides the source code of the Tuner helper APIs for the CapSense CSD 
*  component.
*
* Note:
*
********************************************************************************
* Copyright 2014, Cypress Semiconductor Corporation.  All rights reserved.
* You may use this file only in accordance with the license, terms, conditions,
* disclaimers, and limitations in the end user license agreement accompanying
* the software package with which this file was provided.
*******************************************************************************/

#include "ProgramControl_TunerHelper.h"

#if (ProgramControl_TUNER_API_GENERATE)
    #include "SCB.h"
    #include "SCB_EZI2C.h"
    volatile ProgramControl_MAILBOXES ProgramControl_mailboxesComm;
#endif  /* (ProgramControl_TUNER_API_GENERATE) */

/* `#START ProgramControl_TunerHelper_HEADER` */

/* `#END` */

/*******************************************************************************
* Function Name: ProgramControl_TunerStart
********************************************************************************
*
* Summary:
*  Initializes the CapSense CSD component and EzI2C communication component to use
*  a mailbox data structure for communication with the Tuner GUI.
*  Start the scanning, after initialization is complete.
*
* Parameters:
*  None
*
* Return:
*  None
*
* Global Variables:
*  None
*
*******************************************************************************/
void ProgramControl_TunerStart(void)
{
    #if (0u != ProgramControl_TUNER_API_GENERATE)
		
        /* Init mbx and quick check */
        ProgramControl_InitMailbox(&ProgramControl_mailboxesComm.csdMailbox);
        ProgramControl_mailboxesComm.numMailBoxes = ProgramControl_DEFAULT_MAILBOXES_NUMBER;
        
        /* Start CapSense and baselines */
        ProgramControl_Start();
        
        /* Initialize baselines */
		#if(0u != ProgramControl_CSHL_API_GENERATE)
	        ProgramControl_InitializeEnabledBaselines();
		#endif /* (0u != ProgramControl_CSHL_API_GENERATE) */
		
        /* Start EzI2C, clears buf pointers */
        SCB_Start();
		
        /* Setup EzI2C buffers */
        SCB_EzI2CSetBuffer1(sizeof(ProgramControl_mailboxesComm), sizeof(ProgramControl_mailboxesComm),
                                      			   (volatile uint8 *) &ProgramControl_mailboxesComm);
        
        /* Starts scan all enabled sensors */
        ProgramControl_ScanEnabledWidgets();
    
    #endif  /* (0u != ProgramControl_TUNER_API_GENERATE) */
}


/*******************************************************************************
* Function Name: ProgramControl_TunerComm
********************************************************************************
*
* Summary:
*  This function is blocking. It waits till scanning loop is completed and applies
*  the new parameters from the Tuner GUI if available (the manual tuning mode only). Updates
*  the enabled baselines and state of widgets. Waits while the Tuner GUI reports that 
*  the mailbox content  could be modified. Then loads the report data into the outbox 
*  and sets a busy flag. Starts a new scanning loop.
*  
* Parameters:
*  None
*
* Return:
*  None
*
* Global Variables:
*  None
*
*******************************************************************************/
void ProgramControl_TunerComm(void)
{
    #if (0u != ProgramControl_TUNER_API_GENERATE)
        if (0u == ProgramControl_IsBusy())
        {   
            /* Apply new settings */
            #if((ProgramControl_TUNING_METHOD != ProgramControl__TUNING_NONE) &&\
				(0u != ProgramControl_TOTAL_TRACKPAD_GESTURES_COUNT) ||\
		        (ProgramControl_TUNING_METHOD == ProgramControl__TUNING_MANUAL)) 
     
				ProgramControl_ReadMessage(&ProgramControl_mailboxesComm.csdMailbox);
				
            #endif  /* ((ProgramControl_TUNING_METHOD != ProgramControl__TUNING_NONE) &&\
						(0u != ProgramControl_TOTAL_TRACKPAD_GESTURES_COUNT) ||\
		                (ProgramControl_TUNING_METHOD == ProgramControl__TUNING_MANUAL)) */
			
			#if(0u != ProgramControl_CSHL_API_GENERATE)
				/* Update all baselines and process all widgets */
            	ProgramControl_UpdateEnabledBaselines();
            	ProgramControl_ProcessAllWidgets(&ProgramControl_mailboxesComm.csdMailbox.outbox);
			#endif /* (0u != ProgramControl_CSHL_API_GENERATE) */
			
            ProgramControl_PostMessage(&ProgramControl_mailboxesComm.csdMailbox);
			
            /* Enable EZI2C interrupts, after scan complete */
            SCB_EnableInt();

            while((ProgramControl_mailboxesComm.csdMailbox.type != ProgramControl_TYPE_ID) ||
			      (0u != (SCB_EzI2CGetActivity() & SCB_EZI2C_STATUS_BUSY))){}
            
            /* Disable EZI2C interrupts, while scanning */
            SCB_DisableInt();
            
            /* Start scan all sensors */
            ProgramControl_ScanEnabledWidgets();
        }
    #endif /* (0u != ProgramControl_TUNER_API_GENERATE) */
}


#if ((0u != ProgramControl_TUNER_API_GENERATE) && (0u != ProgramControl_CSHL_API_GENERATE))
    /*******************************************************************************
    * Function Name: ProgramControl_ProcessAllWidgets
    ********************************************************************************
    *
    * Summary:
    *  Calls the required functions to update all the widgets' state:
    *   - ProgramControl_GetCentroidPos() - calls only if linear sliders are
    *     available.
    *   - ProgramControl_GetRadialCentroidPos() - calls only if the  radial slider is
    *     available.
    *   - ProgramControl_GetTouchCentroidPos() - calls only if the  touchpad slider 
    *     available.
    *   - ProgramControl_CheckIsAnyWidgetActive();
    *  The results of operations are copied to OUTBOX.
    *   
    * Parameters:
    *  None
    *
    * Return:
    *  None
    *
    * Global Variables:
    *  ProgramControl_OUTBOX outbox - the structure which is used as an output 
    *  buffer to report data to the Tuner GUI.
    *  Update fields:
    *    - position[];
    *    - OnMask[];
	*
	* Global Variables:
	*  None
	*
    *******************************************************************************/
    void ProgramControl_ProcessAllWidgets(volatile ProgramControl_OUTBOX *outbox)
    {
        uint8  i = 0u;
		
		#if (0u != ProgramControl_TOTAL_MATRIX_BUTTONS_COUNT)
        	uint8 mbPositionBuf[ProgramControl_MB_POSITION_BUFFER_SIZE];
		#endif /* (0u != ProgramControl_TOTAL_MATRIX_BUTTONS_COUNT) */

        #if (0u != ProgramControl_TOTAL_TOUCH_PADS_BASIC_COUNT)
            uint16 centroidPosBuf[ProgramControl_CENTROID_POSITION_BUFFER_SIZE];
        #endif  /* (0u != ProgramControl_TOTAL_TOUCH_PADS_BASIC_COUNT) */
		
		#if(0u != ProgramControl_TOTAL_TRACKPAD_GESTURES_COUNT)
            ProgramControl_POSITION_STRUCT trackpadPosBuf[ProgramControl_CENTROID_POSITION_BUFFER_SIZE];
        #endif  /* (0u != ProgramControl_TOTAL_TRACKPAD_GESTURES_COUNT) */
        
        #if ( (0u != ProgramControl_TOTAL_RADIAL_SLIDERS_COUNT) || (0u != ProgramControl_TOTAL_TOUCH_PADS_COUNT) || \
              (0u != ProgramControl_TOTAL_MATRIX_BUTTONS_COUNT))
            uint8 widgetCnt;
        #endif  /* ((0u != ProgramControl_TOTAL_RADIAL_SLIDERS_COUNT) || (0u != ProgramControl_TOTAL_TOUCH_PADS_COUNT)) || 
                *   (0u != ProgramControl_TOTAL_MATRIX_BUTTONS_COUNT)
                */
        
        /* Calculate widget with centroids */
        #if (0u != ProgramControl_TOTAL_LINEAR_SLIDERS_COUNT)
            for(; i < ProgramControl_TOTAL_LINEAR_SLIDERS_COUNT; i++)
            {
                outbox->centroidPosition[i] = (uint16)ProgramControl_GetCentroidPos((uint32)i);
            }
        #endif /* (0u != ProgramControl_TOTAL_LINEAR_SLIDERS_COUNT) */
        
        #if (0u != ProgramControl_TOTAL_RADIAL_SLIDERS_COUNT)
            widgetCnt = i;
            for(; i < (widgetCnt + ProgramControl_TOTAL_RADIAL_SLIDERS_COUNT); i++)
            {
                outbox->centroidPosition[i] = (uint16)ProgramControl_GetRadialCentroidPos((uint32)i);
            }
        #endif /* (0u != ProgramControl_TOTAL_RADIAL_SLIDERS_COUNT) */
        
        #if (0u != ProgramControl_TOTAL_TOUCH_PADS_BASIC_COUNT)
            widgetCnt = i;
            for(; i < (widgetCnt + (ProgramControl_TOTAL_TOUCH_PADS_BASIC_COUNT * 2u)); i=i+2u)
            {
                if(ProgramControl_GetTouchCentroidPos((uint32)i, centroidPosBuf) == 0u)
                {
                    outbox->centroidPosition[i + ProgramControl_FIRST_FINGER_X_INDEX] = ProgramControl_MAX_UINT_16;
                    outbox->centroidPosition[i + ProgramControl_FIRST_FINGER_Y_INDEX] = ProgramControl_MAX_UINT_16;
                }
                else
                {
                    outbox->centroidPosition[i + ProgramControl_FIRST_FINGER_X_INDEX] =
					(uint16) centroidPosBuf[ProgramControl_FIRST_FINGER_X_INDEX];
                    
					outbox->centroidPosition[i + ProgramControl_FIRST_FINGER_Y_INDEX] =
					(uint16) centroidPosBuf[ProgramControl_FIRST_FINGER_Y_INDEX];
                }
            }
        #endif /* (0u != ProgramControl_TOTAL_TOUCH_PADS_BASIC_COUNT) */
		
		#if (0u != ProgramControl_TOTAL_TRACKPAD_GESTURES_COUNT)
			widgetCnt = i;
			for(; i < (widgetCnt + (ProgramControl_TOTAL_TRACKPAD_GESTURES_COUNT * 4u)); i=i+4u)
			{
				outbox->fingersNum[ProgramControl_DEFAULT_TRACKPAD_INDEX] =
				(uint8)ProgramControl_GetDoubleTouchCentroidPos(trackpadPosBuf);
				
				outbox->gestureID[ProgramControl_DEFAULT_TRACKPAD_INDEX] =
				(uint8)ProgramControl_DecodeAllGestures((uint32)outbox->fingersNum[ProgramControl_DEFAULT_TRACKPAD_INDEX], trackpadPosBuf); 

				#if((0u != ProgramControl_ST_SCROLL_GEST_ENABLED) || (0u != ProgramControl_DT_SCROLL_GEST_ENABLED))
					/*
					Report Scroll count only for scroll and Inertial Scroll gestures.
					For all other gestures report scroll count as zero to tuner.
					 */
					if (((outbox->gestureID[ProgramControl_DEFAULT_TRACKPAD_INDEX]&0xF0u) == 0xC0) ||
							((outbox->gestureID[ProgramControl_DEFAULT_TRACKPAD_INDEX]&0xF0u) == 0xB0u))
					{
						outbox->scrollCnt = ProgramControl_GetScrollCnt();
					}
					else
					{
						outbox->scrollCnt = 0u;
					}
				#endif /* ((0u != ProgramControl_ST_SCROLL_GEST_ENABLED) || (0u != ProgramControl_DT_SCROLL_GEST_ENABLED)) */

                outbox->centroidPosition[i + ProgramControl_FIRST_FINGER_X_INDEX] = ProgramControl_MAX_UINT_16;
                outbox->centroidPosition[i + ProgramControl_FIRST_FINGER_Y_INDEX] = ProgramControl_MAX_UINT_16;
				outbox->centroidPosition[i + ProgramControl_SECOND_FINGER_X_INDEX] = ProgramControl_MAX_UINT_16;
                outbox->centroidPosition[i + ProgramControl_SECOND_FINGER_Y_INDEX] = ProgramControl_MAX_UINT_16;

	            if(0u != outbox->fingersNum[ProgramControl_DEFAULT_TRACKPAD_INDEX])
	            {
	                outbox->centroidPosition[i + ProgramControl_FIRST_FINGER_X_INDEX] =
					(uint16) trackpadPosBuf[ProgramControl_FIRST_FINGER_INDEX].x;
	                
					outbox->centroidPosition[i + ProgramControl_FIRST_FINGER_Y_INDEX] =
					(uint16) trackpadPosBuf[ProgramControl_FIRST_FINGER_INDEX].y;
					
					if(outbox->fingersNum[ProgramControl_DEFAULT_TRACKPAD_INDEX] > 1u)
					{
						outbox->centroidPosition[i + ProgramControl_SECOND_FINGER_X_INDEX] =
						(uint16) trackpadPosBuf[ProgramControl_SECOND_FINGER_INDEX].x;
	                	
						outbox->centroidPosition[i + ProgramControl_SECOND_FINGER_Y_INDEX] =
						(uint16) trackpadPosBuf[ProgramControl_SECOND_FINGER_INDEX].y;
					}
	            }
			}
		#endif /* (0u != ProgramControl_TOTAL_TRACKPAD_GESTURES_COUNT) */

        #if (0u != ProgramControl_TOTAL_MATRIX_BUTTONS_COUNT)
            i += ProgramControl_TOTAL_BUTTONS_COUNT;
            widgetCnt = 0u;
            for(; widgetCnt < (ProgramControl_TOTAL_MATRIX_BUTTONS_COUNT * 2u); widgetCnt += 2u)
            {
                if(ProgramControl_GetMatrixButtonPos((uint32)i, mbPositionBuf) == 0u)
                {
                    outbox->mbPosition[widgetCnt] = ProgramControl_MAX_UINT_8;
                    outbox->mbPosition[widgetCnt+1u] = ProgramControl_MAX_UINT_8;
                }
				else
				{
					outbox->mbPosition[widgetCnt] = mbPositionBuf[0u];
                    outbox->mbPosition[widgetCnt+1u] = mbPositionBuf[1u];
				}
                i += 2u;
            }
        #endif /* (0u != ProgramControl_TOTAL_MATRIX_BUTTONS_COUNT) */

		#if(0u != ProgramControl_CSHL_API_GENERATE)
	        /* Update On/Off State */
	        (void)ProgramControl_CheckIsAnyWidgetActive();

	        /* Copy OnMask */
	        for(i=0u; i < ProgramControl_TOTAL_SENSOR_MASK_COUNT; i++)
	        {
	            outbox->onMask[i]  = ProgramControl_sensorOnMask[i];
	        }
		#endif /* (0u != ProgramControl_CSHL_API_GENERATE) */
		
		/* `#START ProgramControl_ProcessAllWidgets_Debug` */

		/* `#END` */
    }


#endif /* ((0u != ProgramControl_TUNER_API_GENERATE) && (0u != ProgramControl_CSHL_API_GENERATE)) */


/* [] END OF FILE */
